"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var ts = require("typescript");
function isJsOutputFileNewer(inputTsFile) {
    return isTsInputFile(inputTsFile) && isOutputNewer(inputTsFile, ts.Extension.Js);
}
exports.isJsOutputFileNewer = isJsOutputFileNewer;
function isDtsOutputFileNewer(inputTsFile) {
    return isTsInputFile(inputTsFile) && isOutputNewer(inputTsFile, ts.Extension.Dts);
}
exports.isDtsOutputFileNewer = isDtsOutputFileNewer;
function isOutputNewer(inputFile, ext) {
    var outputFilePath = getOutputFileForInput(inputFile, ext);
    try {
        return isFileOlder(inputFile, outputFilePath);
    }
    catch (_a) {
        return false;
    }
}
var mtimeCache = new Map();
function getChangedTime(filePath) {
    var mtime = mtimeCache.get(filePath);
    if (mtime === undefined) {
        mtime = fs.statSync(filePath).mtimeMs;
        mtimeCache.set(filePath, mtime);
    }
    return mtime;
}
function isFileOlder(filePath, thanFilePath) {
    var fileChangedTime = getChangedTime(filePath);
    var thanFileChangedTime = getChangedTime(thanFilePath);
    return thanFileChangedTime > fileChangedTime;
}
function getOutputFileForInput(inputFile, outputExt) {
    var inputExt = path.extname(inputFile);
    return inputFile.substring(0, inputFile.length - inputExt.length) + outputExt;
}
/**
 * @returns true if input file path has .ts or .tsx extension
 */
function isTsInputFile(filePath) {
    if (filePath.endsWith(ts.Extension.Dts)) {
        return false;
    }
    var ext = path.extname(filePath);
    return ext === ts.Extension.Ts || ext === ts.Extension.Tsx;
}
