"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var is_file_newer_1 = require("./is-file-newer");
function patchCompilerHost(host, compilerOptions) {
    if (!compilerOptions.declaration) {
        // if declarations are disabled then we don't need to check if there is newer d.ts file for .ts file
        // in the same way as ts projects does it (medium-sized projects)
        // see https://github.com/Microsoft/TypeScript/issues/3469#issuecomment-400439520
        return;
    }
    // see https://github.com/Microsoft/TypeScript/blob/fcd502502a0d37fa462bd4ae2bd1a67ecee5890b/src/compiler/program.ts#L767-L769
    var moduleResolutionCache = ts.createModuleResolutionCache(host.getCurrentDirectory(), function (x) { return host.getCanonicalFileName(x); });
    host.resolveModuleNames = function (moduleNames, containingFile, reusedNames, redirectedReference) {
        return moduleNames.map(function (moduleName) {
            var resolvedModule = ts.resolveModuleName(moduleName, containingFile, compilerOptions, host, moduleResolutionCache, redirectedReference).resolvedModule;
            // if we have .d.ts file newer than original .ts file
            // we won't compile unchanged .ts files again
            // so let's "fallback" it from .ts to .d.ts
            if (resolvedModule !== undefined && is_file_newer_1.isDtsOutputFileNewer(resolvedModule.resolvedFileName)) {
                resolvedModule.extension = ts.Extension.Dts;
                resolvedModule.resolvedFileName = changeExtension(resolvedModule.resolvedFileName, ts.Extension.Dts);
            }
            return resolvedModule;
        });
    };
}
exports.patchCompilerHost = patchCompilerHost;
function changeExtension(path, newExtension) {
    return ts.changeExtension(path, newExtension);
}
